-- Filename:     mealy_example.vhd
-- Filetype:     VHDL code (behavior)
-- Date:         25 april 2011
-- Update:       1 november 2016
-- Description:  An example of a Mealy FSM
-- Author:       J. op den Brouw
-- State:        demo
-- Error:        -
-- Version:      1.0.2
-- Copyright:    (c)2016, De Haagse Hogeschool

-- This VHDL code describes an example of a Mealy
-- finite state machine. There is not one single
-- method to describe a machine but there are two
-- common description: a 3-process and a 2-process
-- machine. Both are presented here below.

library ieee;
use ieee.std_logic_1164.all;

entity mealy_example is
    port (clk:    in std_logic;
          areset: in std_logic;
          i:      in std_logic;
          u:      out std_logic
         );
end mealy_example;

architecture fsm_2proc of mealy_example is
-- State types
type state_type is (s0, s1);
-- Internal current state and next state
signal current_state, next_state : state_type;
begin
    -- Combinational process generates the
    -- next state and the output
    comb: process (current_state, i) is
    begin
        case current_state is
            when s0 => if i = '0' then
                           next_state <= s0;
                           u <= '0';
                       else
                           next_state <= s1;
                           u <= '1';
                       end if;
            when s1 => if  i = '0' then
                           next_state <= s0;
                           u <= '0';
                       else
                           next_state <= s1;
                           u <= '0';
                       end if;
            when others => next_state <= s0;
                           u <= 'X';
        end case;
    end process;

    -- Sequential process generates the
    -- state register
    reg: process (clk, areset) is
    begin
        if areset = '1' then
            current_state <= s0;
        elsif rising_edge(clk) then
            current_state <= next_state;
        end if;
    end process;

end architecture;

architecture fsm_3proc of mealy_example is
-- State types
type state_type is (s0, s1);
-- Internal current state and next state
signal current_state, next_state : state_type;
begin
    -- Combinational process generates the next state
    combstate: process (current_state, i) is
    begin
        case current_state is
            when s0 => if i = '0' then
                           next_state <= s0;
                       else
                           next_state <= s1;
                       end if;
            when s1 => if  i = '0' then
                           next_state <= s0;
                       else
                           next_state <= s1;
                       end if;
            when others => next_state <= s0;
        end case;
    end process;
  
    -- Sequential process generates the
    -- state register
    reg: process (clk, areset) is
    begin
        if areset = '1' then
            current_state <= s0;
        elsif rising_edge(clk) then
            current_state <= next_state;
        end if;
    end process;

    -- Combinational process generates the output
    combout: process (current_state, i) is
    begin
        case current_state is
            when s0 => if i = '0' then
                           u <= '0';
                       else
                           u <= '1';
                       end if;
            when s1 => u <= '0';
            when others => u <= 'X';
        end case;
    end process;

end architecture;
