-- Filename:     tb_mealy_example.vhd
-- Filetype:     VHDL testbench code
-- Date:         1 november 2016
-- Update:       -
-- Description:  An example of a testbench
-- Author:       J. op den Brouw
-- State:        demo
-- Error:        -
-- Version:      1.1
-- Copyright:    (c)2016, De Haagse Hogeschool

-- This VHDL code describes an example of a testbench
-- for the VHDL description of the Mealy machine in
-- this example project.

library ieee;                                               
use ieee.std_logic_1164.all;                                

-- Testbench has empty entity
entity tb_mealy_example is
end entity tb_mealy_example;

-- The architecture
architecture testbench of tb_mealy_example is
-- Some timing parameters
constant Tperiod : time := 10 ns;
constant small_delay : time := 1 ns;

-- Top level signals to be monitored in the simulator
signal clk : std_logic;
signal i : std_logic;
signal areset : std_logic;
signal u : std_logic;

-- Component description of the Mealy machine
component mealy_example
    port ( clk :    in std_logic;
           areset : in std_logic;
           i :      in std_logic;
           u :      out std_logic
    );
end component;

-- Choose one of the architectures...
--for dut : mealy_example use entity work.mealy_example(fsm_2proc);
for dut : mealy_example use entity work.mealy_example(fsm_3proc);

begin
	
    -- Instantiation of the Mealy machine
    dut : mealy_example
          port map ( clk => clk, i => i, areset => areset, u => u);

    -- The clock, creates a never ending clock
    clockgen: process is
    begin
        clk <= '0';
        wait for Tperiod/2;
        clk <= '1';
        wait for Tperiod/2;
    end process clockgen;

    -- Data generation, including asynchronous reset  
    datagen: process is
    begin
        -- Set known value on the input 
        i<= '0';
		  
        -- Activate reset
        areset <= '1';
        wait for 2*Tperiod;

        -- Deactivate reset
        areset <= '0';
        wait for 2*Tperiod;
		  
		  -- Synchronize on clock edge
        wait until clk = '1';
        wait for small_delay;

        -- Supply the machine with data
        i <= '0';
        wait until clk = '1';
        wait for small_delay;

        i <= '1';
        wait until clk = '1';
        wait for small_delay;

        i <= '1';
        wait until clk = '1';
        wait for small_delay;

        i <= '1';
        wait until clk = '1';
        wait for small_delay;

        i <= '0';
        wait until clk = '1';
        wait for small_delay;

        i <= '1';
        wait until clk = '1';
        wait for small_delay;

        i <= '1';
        wait until clk = '1';
        wait for small_delay;

        i <= '0';
        wait until clk = '1';
        wait for small_delay;

        wait;
    end process datagen;

end testbench;
